class UnsupportedFileType(Exception):
    pass


class UntrustedSource(Exception):
    pass


class MissingParameter(Exception):
    pass


class InapplicableTransformation(Exception):
    pass


class AdapterConversion(Exception):
    pass


class ModelNotFound(Exception):
    pass
